package integration_test_labs

import (
	mnepb "code.fbi.h-da.de/danet/gosdn/api/go/gosdn/networkelement"
	tpb "code.fbi.h-da.de/danet/gosdn/api/go/gosdn/transport"
	integration_test_utils "code.fbi.h-da.de/danet/gosdn/integration-tests/integrationTestUtils"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

func getHostnameSingleMne(mneId string) *mnepb.GetPathRequest {
	gpr := &mnepb.GetPathRequest{
		Timestamp: integration_test_utils.GetTimestamp(),
		Mneid:     mneId,
		Path:      "system/config/hostname",
		Pid:       pndID,
	}

	return gpr
}

func commitOrConfirmChange(cid string, operation mnepb.Operation) *mnepb.SetChangeListRequest {
	sclr := &mnepb.SetChangeListRequest{
		Timestamp: integration_test_utils.GetTimestamp(),
		Change: []*mnepb.SetChange{{
			Cuid: cid,
			Op:   operation,
		}},
		Pid: pndID,
	}
	return sclr
}

func setHostnamePathListRequestSingleMne(mneId string, newHostname string, operation mnepb.ApiOperation) (*mnepb.SetPathListRequest, error) {
	hostnamePath, err := ygot.StringToStructuredPath("/system/config/hostname")
	if err != nil {
		return nil, err
	}

	splr := &mnepb.SetPathListRequest{
		Timestamp: integration_test_utils.GetTimestamp(),
		ChangeRequest: []*mnepb.ChangeRequest{
			{
				Mneid: mneId,
				Path:  hostnamePath,
				Value: &gnmi.TypedValue{
					Value: &gnmi.TypedValue_StringVal{
						StringVal: newHostname,
					},
				},
				ApiOp: operation,
			},
		},
	}
	return splr, nil
}

func createAddListRequestSingleMne(addr, name, id, username, password string) *mnepb.AddListRequest {
	opt := &tpb.TransportOption{
		Address:  addr,
		Username: username,
		Password: password,
		TransportOption: &tpb.TransportOption_GnmiTransportOption{
			GnmiTransportOption: &tpb.GnmiTransportOption{},
		},
		Tls: true,
	}

	alr := &mnepb.AddListRequest{
		Timestamp: integration_test_utils.GetTimestamp(),
		Mne: []*mnepb.SetMne{
			{
				Address:         addr,
				Pid:             pndID,
				PluginId:        integration_test_utils.DefaultTargetPluginUUID,
				TransportOption: opt,
				MneName:         name,
				MneId:           id,
			},
		},
		Pid: pndID,
	}

	return alr
}
